lappend auto_path .
package require nx
package require STORM_i
package provide Clone_Example 0.1

# Example for Functionally Similar Methods (Sect. 3.2)

# Relevant System Fragment
namespace eval ::mtd {

  nx::Class create Clerk {
    :property -accessor public {object}
    :public method checkManagedObjects {} {
      return [[:object get] getBillableAmount]
    }
  }


  nx::Class create AccountManager {
    :property -accessor public {object}
    :public method checkManagedObjects {} {
      return [[:object get] calculateAmount]
    }
  }

  nx::Class create Complex {
    :property -accessor public {object}
    :public method calculateExpenses {} {
      return [[:object get] calculateAmount]
    }
  }

  nx::Class create LifelineSite {
    :property siteConf:object,type=::mtd::SiteConfig,required
    :public method getBillableAmount {} {
      set base [expr {[${:siteConf} units get] * [${:siteConf} rate get] * 0.5}]
      set t [Tax new]
      set taxRate [$t standardTaxRate get]
      set tax [expr {$base * [$t getIndividualTaxRate lifeline] * $taxRate / 10}]
      return [expr {$base + $tax}]
    }
  }

  nx::Class create ResidentialSite {
    :property siteConf:object,type=::mtd::SiteConfig,required
    :public method calculateAmount {} {
      set t [Tax new]
      set taxRate [$t standardTaxRate get]
      set base [expr {[${:siteConf} units get] * [${:siteConf} rate get]}]
      set tax [expr {$base * [$t getIndividualTaxRate residential] * $taxRate / 10}]
      return [expr {$base + $tax}]
    }

    :public method calculateServiceFee {value} {
      if {$value > 5} {
        return 480
      } else {
        return 180
      }
    }
  }

  nx::Class create IndustrialSite {
    :property siteConf:object,type=::mtd::SiteConfig,required
    :public method calculateAmount { } {
      set t [Tax new]
      set taxRate [$t standardTaxRate get]
      set base [expr {[${:siteConf} units get] * [${:siteConf} rate get] * $taxRate / 10}]

      set support [:calculateStateSupport $base]
      return [expr {$base - $support}]
    }

    :public method calculateStateSupport {value} {
      if {$value > 10} {
        return [expr {0.3 * $value}]
      } else {
        return 0
      }
    }
  }

  nx::Class create Tax {
    :property -accessor public {standardTaxRate:integer 4}

    :public method getIndividualTaxRate {{type default}} {
      array set "" {
        residential 12
        lifeline 7
        default 0
      }
      return $($type)
    }

  }

  nx::Class create SiteConfig {
    :property -accessor public {rate:integer 5}
    :property -accessor public {units:integer 10000}
  }

  namespace export SiteConfig ResidentialSite LifelineSite IndustrialSite Complex Clerk AccountManager

}


# Relevant Test Scenarios
namespace import ::mtd::*

set c1 [::STORM_i::TestCase new -name Case1]

$c1 setup_script set {
  set x [ResidentialSite new -siteConf [SiteConfig new]]
}

set s1c1 [::STORM_i::TestScenario new -name scen1c1 -testcase Case1]
$s1c1 test_body set {
  set co [Complex new]
  set sc [SiteConfig new]
  $co object set [ResidentialSite new -siteConf $sc]
  $co calculateExpenses
}

set s2c1 [::STORM_i::TestScenario new -name scen2c1 -testcase Case1]
$s2c1 test_body set {
  set sc [SiteConfig new]
  set cl [Clerk new]
  $cl object set [LifelineSite new -siteConf $sc]
  $cl checkManagedObjects
}

set s3c1 [::STORM_i::TestScenario new -name scen3c1 -testcase Case1]
$s3c1 test_body set {
  set sc [SiteConfig new]
  set am [AccountManager new]
  $am object set [IndustrialSite new -siteConf $sc]
  $am checkManagedObjects

}

set tests [::STORM_i::TestSuite new -detailed_report 1 -name Tests]
$tests order set [lsort -unique [::STORM_i::TestCase info instances]]
$tests runAllTestCases

# Local variables:
#    mode: tcl
#    tcl-indent-level: 2
#    indent-tabs-mode: nil
# End:
