lappend auto_path .
package require nx 
package require STORM_i
#package require VM
package provide Stack_Example 0.1
################################
# Simple Example: Stack
################################

################################################################
# class Stack with operations push, pop, top, size and full
# and the attribute limit 
################################################################
nx::Class create Stack {
  :property -accessor public {limit:integer 4 }
  :property -accessor public {element {} }
  
  :public object method getInstance {} {
    if {[llength [:info instances]] == 0} {
      set s [Stack new]
      return $s
    } elseif {[llength [:info instances]] == 1} {
      return [:info instances]
    } elseif {[llength [:info instances]] > 1} {
      puts "\033\[31merror\033\[0m"
      return [lindex [:info instances] 0]
    } else {puts "\033\[31merror\033\[0m"}
  } 

  :public method size { } {
    return [llength [:element get]]    
  }
	
  :public method full { } {
    set full 0
    if {[:limit get] <= [:size]} {
      set full 1   
    }
    return $full        
  }
	
  :public method push {e} {
    if {[:full]} {
      return 0
    } else {
      if { [:checkElement $e] } {
        :element set [concat [list $e] [:element get]]
        return 1
      } else { return 0 }
    }	    
  }	
	
  :public method pop { } {
    if {[:size] == 0} {
      return 0
    } else {
      set top [:top]
      :element set [lrange [:element get] 1 end]
      return $top	
    }
  }
	
  :public method top { } {
    set top [lindex [:element get] 0]
    return $top				
  }
	
  :public method checkElement {e} {
    if {[$e info has type ::Element]} {
      return 1
    } else { return 0 }
	  
  }
}

################################################
# class Element with attributes name and value
################################################
nx::Class create Element {
  :property -accessor public {name}
  :property -accessor public {value} 
}

nx::Class create OtherClass { }


################################################################
# scenario-test script
# with test cases pushElement and popElement
################################################################


################################################################
# test case pushElement
################################################################
set pE [::STORM_i::TestCase new -name PushElement]

$pE setup_script set {
  set s1 [::Stack getInstance]
  $s1 push [::Element new -name e1 -value 3.5]
}

$pE preconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
}

$pE postconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
  {expr {[[::Stack getInstance] limit get] == 4}}
}



################################
# test scenario pushOnFullStack
################################
set pF [::STORM_i::TestScenario new -name pushOnFullStack -testcase PushElement]

$pF expected_result set 0

$pF setup_script set {
  [::Stack getInstance] limit set 1
}

$pF preconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
  {expr {[[::Stack getInstance] limit get] == 1}}
}

$pF test_body set {
  [::Stack getInstance] push [::Element new -name e2 -value 1.4] 
}

$pF postconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
}

$pF cleanup_script set {
  [::Stack getInstance] limit set 4
}


##################################
# test scenario pushElementDefault
##################################
set pD [::STORM_i::TestScenario new -name pushElementDefault -testcase PushElement]

$pD expected_result set 1

$pD setup_script set {
  [::Stack getInstance] limit set 5
}

$pD preconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
  {expr {[[::Stack getInstance] limit get] == 5}}
}

$pD test_body set {
  [::Stack getInstance] push [::Element new -name e3 -value 1.8] 
  [::Stack getInstance] push [::Element new -name e4 -value 2.6]
}

$pD postconditions set {
  {expr {[[::Stack getInstance] size] == 3}}
  {expr {[[[::Stack getInstance] top] name get] eq "e4"}}
  {expr {[[[::Stack getInstance] top] value get] == 2.6}}
}

$pD cleanup_script set {
  [::Stack getInstance] pop
  [::Stack getInstance] pop
  [::Stack getInstance] limit set 4
}


#################################
# test scenario pushOnEmptyStack
#################################
set pES [::STORM_i::TestScenario new -name pushOnEmptyStack -testcase PushElement]

$pES expected_result set 1

$pES setup_script set {
  [::Stack getInstance] pop
}

$pES preconditions set {
  {expr {[[::Stack getInstance] size] == 0}}
  {expr {[[::Stack getInstance] limit get] == 4}}
}

$pES test_body set {
  [::Stack getInstance] push [::Element new -name e5 -value 1.9]
}

$pES postconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
  {expr {[[[::Stack getInstance] top] name get] eq "e5"}}
  {expr {[[[::Stack getInstance] top] value get] == 1.9}}
}

$pES cleanup_script set {
  [::Stack getInstance] limit set 4
}

###################################
# test scenario pushInvalidElement
###################################
set pIE [::STORM_i::TestScenario new -name pushInvalidElement -testcase PushElement]

$pIE expected_result set 0

$pIE preconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
}

$pIE test_body set {
  [::Stack getInstance] push [::OtherClass new]	
}

$pIE postconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
}


##############################################################
# test case popElement
##############################################################
set popE [::STORM_i::TestCase new -name PopElement]

$popE preconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
}

$popE postconditions set {
  {expr {[[::Stack getInstance] size] == 1}}
}

##################################
# test scenario popElementDefault
##################################
set popED [::STORM_i::TestScenario new -name popElementDefault -testcase PopElement]

$popED setup_script set {
  [::Stack getInstance] push [::Element new -name e6 -value 3.1]
  [::Stack getInstance] push [::Element new -name e7 -value 4.6]
}

$popED preconditions set {
  {expr {[[::Stack getInstance] size] == 3}}

}

$popED test_body set {
  [::Stack getInstance] pop
}

$popED postconditions set {
  {expr {[[::Stack getInstance] size] == 2}}
  {expr {[[[::Stack getInstance] top] name get] eq "e6"}}
  {expr {[[[::Stack getInstance] top] value get] == 3.1}}
  {expr {[[::Stack getInstance] pop] != ""}}
}

#######################################
#test scenario popElementFromEmptyStack
#######################################
set popEES [::STORM_i::TestScenario new -name popElementFromEmptyStack -testcase PopElement]

$popEES expected_result set 0

$popEES setup_script set {
  [::Stack getInstance] pop
  [::Stack getInstance] pop
}

$popEES preconditions set {
  {expr {[[::Stack getInstance] size] == 0}}
}

$popEES test_body set {
  [::Stack getInstance] pop
}

$popEES postconditions set {
  {expr {[[::Stack getInstance] size] == 0}}
  {expr {[[::Stack getInstance] top] == ""}}
}

$popEES cleanup_script set {
  [::Stack getInstance] push [::Element new -name e8 -value 0.3]
}
