lappend auto_path .
#pkg_mkIndex . storm/* traceProvider/* viewConfig/* modelBuilder/* qsdExport/* ksTests/*
package require Tk
package require nx
package require QXP
package require TR
package require TA
package require STORM_i
package require VM
package require MB
#package require IMV


# UI for KaleidoScope v0.3



########################
# Basic Configuration
########################

ttk::style theme use alt

wm title . "KaleidoScope"
#wm geometry . 1400x1000+20+20
# set size of application to maximum
set screenheight [winfo screenheight .]
set screenwidth [winfo screenwidth .]
wm geometry . ${screenwidth}x${screenheight}+0+0
#. configure -background "grey"
grid [ttk::frame .c -padding "3 3 5 5"] -column 0 -row 0 -sticky n

########################
# Sect. Test Execution
########################
grid [ttk::labelframe .c.te -text "Test Execution"] -sticky w

# button: test-script selection
grid [ttk::button .c.te.select -text "Select STORM test script" -command selectTestScript -width 20] -sticky w
grid [ttk::label .c.te.selectedPath -textvariable path] -sticky w

# select a test script (and implicitely the corresponding SUT)
proc selectTestScript {} {
	# pop dialog for selection
	set pathSelect [tk_getOpenFile -initialdir ./examples -title "Select a test-script file"]
	global testScript
	global testScriptContent
	if {$pathSelect eq ""} {
		set ::path "No test script selected"
		set testScript ""
	} else {
		set ::path $pathSelect
		set testScript $pathSelect
		set ::error ""

		# get the content of the test script
		set f [open $testScript r]
		set testScriptContent [read $f]
		close $f

		# enable button trigger test execution
		.c.te.triggerRun configure -state activ

		# write content into text field .nb.f1.t
		# and set again to readonly
		.nb.f1.t configure -state normal
		.nb.f1.t delete 1.0 end
		.nb.f1.t insert end $testScriptContent
		.nb.f1.t configure -state disabled
		.nb select .nb.f1
	}
}

# button: trigger test execution
grid [ttk::button .c.te.triggerRun -text "Trigger test execution" -command triggerTestExecution -width 20 -state disabled] -sticky w
grid [ttk::label .c.te.error -textvariable error] -sticky w

# trigger the exeuction of the selected test script
proc triggerTestExecution {} {
	# TODO clear cache (here or after test execution)

	global testScript
	if {$testScript eq ""} {
		set ::error "Please select a test-script file"
	} else {
		set ::error ""

		# TODO check whether file is STORM test script
		# currently: check whether file type is .tcl
 		set fileType [string range $testScript end-3 end]
		if {![expr {$fileType eq ".tcl"}]} {
			set ::error "Please select a STORM test-script file (*.tcl)"
		} else {
			set ::path ""
			set ::error ""
			# create folder with time stamp
			set currentName [clock seconds]
			file mkdir files/tmp

			# retrieve the test-script content from the text field
			# (allows for manipulating the content (in the future))
			#set testScriptContent [.nb.f1.t get 1.0 {end -1c}]

			# executing the test script and retrieving the test result
			set testResult [source $testScript]

			# write test result into text field .nb.f2.t
			# and set again to readonly
			.nb.f2.t configure -state normal
			.nb.f2.t delete 1.0 end
			.nb.f2.t insert end $testResult
			.nb.f2.t configure -state disabled
			.nb select .nb.f2

			# if trace model exists
			.c.stm.inputTraceModelName configure -state active
			set traceModelNameDefault tm_$currentName
			set ::tmName $traceModelNameDefault
			.c.stm.triggerTransformation configure -state active
		}
	}
}

########################
# Sect. Save Trace Model
########################
grid [ttk::labelframe .c.stm -text "Save Trace Model"] -column 0 -row 2 -sticky w
grid [ttk::entry .c.stm.inputTraceModelName -state disabled -textvariable tmName] -sticky w
# button: save extracted trace model
grid [ttk::button .c.stm.triggerTransformation -text "Save Trace Model" -command saveTraceModel -width 20 -state disabled] -sticky w
grid [ttk::label .c.stm.error -textvariable tmMessage] -sticky w

# save the extracted model under the defined name
proc saveTraceModel {} {
	global currentName
	if {[.c.stm.inputTraceModelName get] eq ""} {
		set ::tmMessage "Invalid name"
	} else {

		#	read content from files/tmp/traceModel.xmi
		# (temporary folder tmp; TODO: delete after model transformation)
		set f [open files/tmp/traceModel.xmi r]
		set tmContent [read $f]
		close $f
		# create file files/traceModels/tm_${currentName}.xmi

		# create new trace model file and write the corresponding content
		set fileId [open files/traceModels/[.c.stm.inputTraceModelName get].xmi "w"]
		puts -nonewline $fileId $tmContent
		close $fileId

		# collect and display all available trace models for trace selection
		set tms [glob -directory files/traceModels *.xmi]
		# get the root names of the trace models (without paths)
		set tsShort ""
		foreach t $tms {
			lappend tsShort [file rootname [file tail $t]]
		}
		# sort the models alphanumerical
		set tsShort [lsort -decreasing $tsShort]

		.c.ts.selectTraceModel configure -values $tsShort
		.c.ts.selectTraceModel current 0

		set ::tmMessage "\"[.c.stm.inputTraceModelName get]\" saved!"

		# cleanup
		set ::tmName ""
		#exec rm -R files/tmp
	}
}

########################
# Sect. Trace Selection
########################
grid [ttk::labelframe .c.ts -text "Trace Selection"] -column 0 -row 3 -sticky w

# collect and display all available trace models for trace selection
set tms [glob -directory files/traceModels *.xmi]
# get the root names of the trace models (without paths)
set tsShort ""
foreach t $tms {
	lappend tsShort [file rootname [file tail $t]]
}
# sort the models alphanumerical
set tsShort [lsort -decreasing $tsShort]

grid [ttk::combobox .c.ts.selectTraceModel -values $tsShort -state readonly -width 20] -sticky w
.c.ts.selectTraceModel current 0

grid [ttk::button .c.ts.goToConfiguration -text "Go to view configuration" -command goToConfig -width 20] -sticky w

# TODO bind message to trace selection that resets view configuration to default values and sets it to readonly state!!


# got to view configuration
proc goToConfig {} {

	# clean up
	set ::tmMessage ""

	# check the selected trace model
	set tmPath files/traceModels/[.c.ts.selectTraceModel get].xmi

	global ta
	set ta [::TA::TraceAnalyzer new]
	$ta setTraceModel $tmPath

	# extract the given test parts and list them in corresponding select boxes
	# get all test cases
	global testCases
	set testCases [$ta getCases [$ta tmd get]]
	set testCases [linsert $testCases 0 all]
	.c.vc.selectTestCase configure -values $testCases
	.c.vc.selectTestCase current 0

	# default is 'all' (scenarios in dependency of test case; see below)
	global testScenarios
	set testScenarios [linsert $testScenarios 0 all]
	.c.vc.selectTestScenario configure -values $testScenarios
	.c.vc.selectTestScenario current 0

	# get all test blocks of test suite
	global testBlocks
	set testBlocks [$ta getSuiteBlocks [$ta tmd get]]
	set testBlocks [linsert $testBlocks 0 all]
	.c.vc.selectTestBlock configure -values $testBlocks
	.c.vc.selectTestBlock current 0
}

###########################
# Sect. View Configuration
###########################
grid [ttk::labelframe .c.vc -text "View Configuration"] -column 0 -row 4 -sticky w

set modelTypes { "UML sequence diagram" "UML inter component-interaction specification" }
grid [ttk::combobox .c.vc.selectModelType  -values $modelTypes -state readonly] -sticky w
.c.vc.selectModelType current 0

set isEntireTestSuite 0
grid [ ttk::checkbutton .c.vc.isEntireTestSuite -text "Entire Test Suite" -variable isEntireTestSuite] -sticky w

proc isCheckedIsEntireTestSuite {} {
	global isEntireTestSuite
	# from 1 to 0
	if {$isEntireTestSuite} {
		#puts toBeEntire0
		# TODO enable modification
		.c.vc.selectTestCase current 0
		.c.vc.selectTestCase configure -state readonly
		.c.vc.selectTestScenario current 0
		.c.vc.selectTestScenario configure -state readonly
		.c.vc.selectTestBlock current 0
		.c.vc.selectTestBlock configure -state readonly

	# from 0 to 1
	} else {
		#puts toBeEntire1
		# TODO set all part select boxes to 'all' and disable modification (readonly)
		.c.vc.selectTestCase current 0
		.c.vc.selectTestCase configure -state disabled
		.c.vc.selectTestScenario current 0
		.c.vc.selectTestScenario configure -state disabled
		.c.vc.selectTestBlock current 0
		.c.vc.selectTestBlock configure -state disabled

	}
}
bind .c.vc.isEntireTestSuite <ButtonRelease-1> [list isCheckedIsEntireTestSuite]

set testCases {}
grid [ttk::combobox .c.vc.selectTestCase -values $testCases -state readonly] -sticky w

# modify values of select boxes triggered by change in selectTestCase
proc caseSelected {} {
	global ta

	if {[.c.vc.selectTestCase get] eq "all"} {
		set testScenarios all
		set testBlocks [$ta getSuiteBlocks [$ta tmd get]]
	} else {
		set testScenarios [$ta getCaseScenarios [$ta tmd get] [.c.vc.selectTestCase get]]
		set testBlocks [$ta getCaseBlocks [$ta tmd get] [.c.vc.selectTestCase get]]
	}
	set testScenarios [linsert $testScenarios 0 all]
	.c.vc.selectTestScenario configure -values $testScenarios
	.c.vc.selectTestScenario current 0
	set testBlocks [linsert $testBlocks 0 all]
	.c.vc.selectTestBlock configure -values $testBlocks
	.c.vc.selectTestBlock current 0
}
# bind the method caseSelected to selectTestCase (ChangeListener)
bind .c.vc.selectTestCase <<ComboboxSelected>> [list caseSelected]

set testScenarios {}
grid [ttk::combobox .c.vc.selectTestScenario -values $testScenarios -state readonly] -sticky w

# modify values of select boxes triggered by change in selectTestScenario
proc scenarioSelected {} {

		global ta
		set testBlocks [$ta getScenarioBlocks [$ta tmd get] PushElement pushOnEmptyStack]
		set testBlocks [linsert $testBlocks 0 all]
		.c.vc.selectTestBlock configure -values $testBlocks
		.c.vc.selectTestBlock current 0
}
# bind the method scenarioSelected to selectTestScenario (ChangeListener)
bind .c.vc.selectTestScenario <<ComboboxSelected>> [list scenarioSelected]

set testBlocks {}
grid [ttk::combobox .c.vc.selectTestBlock -values $testBlocks -state readonly] -sticky w

set featureCallScopes { "sutIntern" "stfToSut" all}
grid [ttk::combobox .c.vc.selectFeatureCallScope -values $featureCallScopes -state readonly] -sticky w
.c.vc.selectFeatureCallScope current 0

grid [ttk::entry .c.vc.inputModelName -textvariable umlModelName] -sticky w

# optional description of the configured view (drawn on the resulting diagram)
set includesDescription 0
grid [ ttk::checkbutton .c.vc.includesDescription -text "Description" -variable includesDescription] -sticky w

# button: trigger transformation
grid [ttk::button .c.vc.triggerTransformation -text "Create UML model" -command triggerModelTransformation -width 20] -sticky w
grid [ttk::label .c.vc.error -textvariable umlMessage] -sticky w


# trigger the m2m transformation
proc triggerModelTransformation {} {
	global umlModelName
	if {[.c.vc.inputModelName get] eq ""} {
		set ::umlMessage "Invalid name"
	} else {

		# set name of the resulting UML model
		set umlModelName [.c.vc.inputModelName get]

		# change isEntireTestSuite from 0/1 to true/false (for subsequent usage in view model)
		global isEntireTestSuite
		if {$isEntireTestSuite} {
			set isEntireTestSuite true
		} else {
			set isEntireTestSuite false
		}

		# create view model
		set vmc [::VM::VMCreator new]
		$vmc createXMIModel vm [.c.vc.selectFeatureCallScope get] [.c.vc.selectTestCase get] [.c.vc.selectTestScenario get] [.c.vc.selectTestBlock get] $isEntireTestSuite

		# create description based on view configuration
		global includesDescription
		set description ""
		if {$includesDescription} {
			lappend description [.c.ts.selectTraceModel get] $isEntireTestSuite [.c.vc.selectTestCase get] [.c.vc.selectTestScenario get] [.c.vc.selectTestBlock get] [.c.vc.selectFeatureCallScope get]
		}

		# trigger QVTo model transformation
		set mb [::MB::ModelBuilder new]
		set v files/tmp/vm.xmi

		set tm files/traceModels/[.c.ts.selectTraceModel get].xmi

		#set tm files/tmp/traceModel.xmi
		$mb createInteractionModel $v $tm out.uml

		# get XMI content from out.uml
		set f [open out.uml r]
		set umlModelContent [read $f]
		close $f

		# write content into text field .nb.f3.t
		.nb.f3.t configure -state normal
		.nb.f3.t delete 1.0 end
		.nb.f3.t insert end $umlModelContent
		.nb.f3.t configure -state disabled
		.nb select .nb.f3

		# write content into file
		set fileId [open files/umlModels/$umlModelName "w"]
		#puts -nonewline $fileId $testScriptContent
		puts $fileId $umlModelContent
		close $fileId

		#[::QXP::QsdExporter new] printDiagram out.uml $umlModelName png
		# with description
		[::QXP::QsdExporter new] printDiagram out.uml $umlModelName png $description

		# Show diagram on canvas
		showDiagram ${umlModelName}.png

		# TODO cleanup
		#exec rm -R files/tmp
	}
}

# display the diagram on canvas
proc showDiagram {sdFile} {

	# large example
	#set sdFile "largeT2.png"

	# clean canvas
	.nb.f4.c delete dia

	# create new image
	set sdia [image create photo]
	$sdia read $sdFile

	# TODO scaling large images
	#set $sd [scaleImage $sd 0.6]

	# display image on canvas
	.nb.f4.c create image 0 0 -image $sdia -anchor nw -tags dia
	.nb select .nb.f4

	# set scroll region to image size
	set imgWidth [image width $sdia]
	set imgHeight [image height $sdia]
	.nb.f4.c configure -scrollregion "0 0 $imgWidth $imgHeight"

}

# resizing an image
proc scaleImage {im xfactor {yfactor 0}} {
    set mode -subsample
    if {abs($xfactor) < 1} {
       set xfactor [expr round(1./$xfactor)]
    } elseif {$xfactor>=0 && $yfactor>=0} {
        set mode -zoom
    }
    if {$yfactor == 0} {set yfactor $xfactor}
    set t [image create photo]
    $t copy $im
    $im blank
    $im copy $t -shrink $mode $xfactor $yfactor
    image delete $t
 }

###########################
# Sect. Image-File Export
###########################
grid [ttk::labelframe .c.xd -text "Export Diagram"] -column 0 -row 5 -sticky w

set fileTypes { "svg" "png" "pdf" "ps"}
grid [ttk::combobox .c.xd.selectFileType  -values $fileTypes -state readonly] -sticky w
.c.xd.selectFileType current 0
grid [ttk::button .c.xd.createImageFile -text "Create Image File" -command createImageFile -width 20] -sticky w

grid [ttk::label .c.xd.exportPath -textvariable exportPath] -sticky w


# trigger the execution of qsd to render a given .sd file into the selected diagram format
proc createImageFile {} {
	global umlModelName

	# set sd input file
	# TODO not from root directory!
	set inputFile ${umlModelName}.sd

	set outputPath files/imgs/$umlModelName.[.c.xd.selectFileType get]
	set fileType [.c.xd.selectFileType get]
	set ::exportPath "Image file created: $outputPath"

	# trigger the image creation in QSD (resulting file name, resulting type, and path of the input file)
	[::QXP::QsdExporter new] triggerQsdPrint files/imgs/$umlModelName $fileType $inputFile

	# open the file with default appplication
	exec xdg-open $outputPath
}

################################################################################
# Frame Organization

grid [ttk::notebook .nb] -column 1 -row 0 -sticky n
	.nb add [frame .nb.f1] -text "Test Script"
	#ttk::scrollbar .nb.f1.vsb -orient vertical -command [list .nb.f1 yview]

	.nb select .nb.f1
	.nb add [frame .nb.f2] -text "Test Log"
	.nb add [frame .nb.f3] -text "UML Interaction Model (XMI)"
	.nb add [frame .nb.f4] -text "UML Sequence Diagram"
grid [tk::text .nb.f1.t -width 200 -height 70 -background white]
#-yscrollcommand [list .nb.f4.vsb set]]
#grid .nb.f1.vsb -row 0 -column 1 -sticky nsew

#grid columnconfigure .nb.f1 0 -weight 1
#grid rowconfigure .nb.f1 0 -weight 1

grid [tk::text .nb.f2.t -width 200 -height 70 -background white]
grid [tk::text .nb.f3.t -width 200 -height 70 -background white]
grid [tk::label .nb.f4.l -width 0 -height 0]
.nb.f1.t configure -state disabled
.nb.f2.t configure -state disabled
.nb.f3.t configure -state disabled


ttk::scrollbar .nb.f4.hsb -orient horizontal -command [list .nb.f4.c xview]
ttk::scrollbar .nb.f4.vsb -orient vertical -command [list .nb.f4.c yview]
canvas .nb.f4.c -xscrollcommand [list .nb.f4.hsb set] -yscrollcommand [list .nb.f4.vsb set] -background white
grid .nb.f4.c -row 0 -column 0 -sticky nsew
grid .nb.f4.vsb -row 0 -column 1 -sticky nsew
grid .nb.f4.hsb -row 1 -column 0 -sticky nsew
grid columnconfigure .nb.f4 0 -weight 1
grid rowconfigure .nb.f4 0 -weight 1

# dragging the convas with the mouse
bind .nb.f4.c <ButtonPress-1> {%W scan mark   %x %y}
bind .nb.f4.c <B1-Motion>     {%W scan dragto %x %y 1}

#########################################################################################################

#.nb.f1.t insert 1.0 $testScript

#grid columnconfigure . 0 -weight 20
#grid columnconfigure . 1 -weight 80
#grid rowconfigure . 0 -weight 20
#grid rowconfigure . 1 -weight 20
#grid rowconfigure . 2 -weight 20
foreach w [winfo children .c] {grid configure $w -padx 10 -pady 15}
#bind . <Return> {triggerTestRun}

################################################################################

# globals
set currentName ""
set testScript ""
set traceModelXmi ""
set umlModelXmi ""
set umlModelName ""
set viewModelXmi ""
set sdFile ""
set testScriptContent ""
set ta ""
