package require nx
package require tdom
package provide QXP 0.1

namespace eval ::QXP {
  ##################################
  # class QSDExporter
  ##################################
  nx::Class create QsdExporter {

    :property -accessor public {titleInput}
    :property -accessor public {lifelineInput}
    :property -accessor public {messageInput}
    #:property -accessor public {diagramName}
    :property -accessor public {descriptionInput}

    # Wrapping function for printing a diagram
    # by passing the path of the XMI file, the name and type of the resulting diagramName
    # A interem result is the QSD input file (<diagramName>.sd)
    #:public method printDiagram { xmiFile diagramName fileType}
    :public method printDiagram { xmiFile diagramName fileType description} {
      #set f exampleModel1.xmi
      #:diagramName set $diagramName
      #set inputFile [:getQsdInput $xmiFile $diagramName]
      set inputFile [:getQsdInput $xmiFile $diagramName $description]
      #:triggerQsdPrint "$diagramName.$fileType" $fileType $inputFile
      :triggerQsdPrint $diagramName $fileType $inputFile
    }

    # Trigger the QSD rendering by passing the path and type of the output file
    # and the QSD input file (*.sd)
    :public method triggerQsdPrint { diagramName outputType inputFile} {
      set typeList {svg pdf png ps}
      if {[lsearch $typeList $outputType] >= 0} {
        puts [exec java -jar qsdExport/sdedit-4.2-beta8.jar -o "$diagramName.$outputType" -t $outputType $inputFile];# 'java -jar' added due to 'invalid file (bad magic number): Exec format error'
        #puts "Diagram file created"
      } else {
        puts "Error: output type \'$outputType\' not supported! Please select svg, pdf, png, or ps"
      }
    }

    # Convert a given XMI file into QSD input file with given name (<name>.sd)
    :protected method getQsdInput {xmiFile name description} {
      set xmi [read [open $xmiFile r]]
      set qsdInput ""

      :titleInput set [:exportTitle $name]
      :lifelineInput set [:exportLifelines $xmi]
      :messageInput set [:exportMessages $xmi]

      :descriptionInput set ""
      if {$description ne ""} {
        :descriptionInput set [:exportDescription [lindex $description 0] [lindex $description 1] [lindex $description 2] [lindex $description 3] [lindex $description 4] [lindex $description 5]]
      }


      append qsdInput [:titleInput get]
      append qsdInput [:descriptionInput get]
      append qsdInput [:lifelineInput get]
      append qsdInput "\n";#important!
      #append qsdInput [:exportMessages $xmi]
      append qsdInput [:messageInput get]
      :createFile $name $qsdInput
    }

    :protected method exportDescription {traceModelName entireTS testCase testScenario testBlock callScope} {
      set descrText ""
      append descrText "#!>> \n"
      append descrText "#!Configured view based on trace model '$traceModelName': \n"
      append descrText "#!TestCase '$testCase', TestScenario '$testScenario', TestBlock '$testBlock', \n"
      append descrText "#!EntireTS '$entireTS', CallScope '$callScope' \n"
      append descrText "#!<< \n"
      return $descrText
    }

    # Create QSD input text for title and frame
    :protected method exportTitle {name} {
      return "#!\[sd $name \]  \n"
    }

    # Create QSD input text for lifelines
    :protected method exportLifelines {fileData} {
      set llText ""

      set doc [dom parse $fileData]
      set root [$doc documentElement]
      set pElements [$root selectNodes packagedElement]
      foreach pe $pElements {
        if {[$pe getAttribute xmi:type] eq "uml:Interaction"} {
          foreach l [$pe selectNodes lifeline] {

            set objectName ""
            set className ""
            set lifelineText ""
            set o [$root selectNodes "//ownedAttribute\[@*\[local-name()='id'\] ='[$l getAttribute represents]'\]"]
            set objectName [$o getAttribute name]
            set c [$root selectNodes "//packagedElement\[@*\[local-name()='id'\] ='[$o getAttribute type]'\]"]

            # TestDriver lifeline is anonymous ([a])
            if {[$c getAttribute name] eq "TestDriver"} {
              set className "[$c getAttribute name] \[a\]"
            } elseif {[$c getAttribute name] ne ""} {
              set className [$c getAttribute name]
            } else {
              set className nxClass
            }
            append lifelineText $objectName ":" $className
            append llText $lifelineText " \n"
          }
        }
      }
      return $llText
    }

    # Create QSD input text for messages
    :protected method exportMessages {fileData} {
      set msText ""

      set messageSent ""
      set tmpCalleeOld ""
      set callLevelCounter 0
      set chainCaller ""
      set callLevel 0
      set doc [dom parse $fileData]
      set root [$doc documentElement]
      set pElements [$root selectNodes packagedElement]
      foreach pe $pElements {
        if {[$pe getAttribute xmi:type] eq "uml:Interaction"} {

          foreach f [$pe selectNodes fragment] {
            if {[$f getAttribute xmi:type] eq "uml:MessageOccurrenceSpecification"} {
              set fID [$f getAttribute xmi:id]

              set messageText ""

              # get message name
              set mID [$f getAttribute message]
              set m [$root selectNodes "//message\[@*\[local-name()='id'\] ='$mID'\]"]
              set mName [$m getAttribute name]

              set args ""
              set accessorTag ""

              # catching the case: "info has type"
              if {$mName eq "info"} {
                foreach a [$m selectNodes argument] {
                  if {[$a getAttribute name] eq "has" || [$a getAttribute name] eq "type"} {
                    append mName _[$a getAttribute name]
                  } else {
                    lappend args [$a getAttribute name]
                  }
                }

              } else {

                foreach a [$m selectNodes argument] {
                  # catching accessors get and set
                  if {[$a getAttribute name] eq "get"} {
                    set accessorTag <<getter>>
                  } elseif {[$a getAttribute name] eq "set"} {
                    set accessorTag <<setter>>

                  # default case of "normal" passed method arguments
                  } else {
                    lappend args [$a getAttribute name]
                  }
                }
              }
              set argsText ([join $args ", "])
              #puts $argsText

              # get id of sending event
              set seID [$m getAttribute sendEvent]
              set sE [$root selectNodes "//fragment\[@*\[local-name()='id'\] ='$seID'\]"]

              # only consider send events and the sent messages
              # (in order to avoid handling messages twice)
              #puts "compare: $fID $seID"
              if {$fID eq $seID} {

                # get source name
                set sourceID [$sE getAttribute covered]
                set source [$root selectNodes "//lifeline\[@*\[local-name()='id'\] ='$sourceID'\]"]
                set sourceName [$source getAttribute name]
                # caller
                set callerName [$sE getAttribute name]

                # get target name
                set reID [$m getAttribute receiveEvent]
                set rE [$root selectNodes "//fragment\[@*\[local-name()='id'\] ='$reID'\]"]
                set targetID [$rE getAttribute covered]
                set target [$root selectNodes "//lifeline\[@*\[local-name()='id'\] ='$targetID'\]"]
                set targetName [$target getAttribute name]
                # callee
                set calleeName [$rE getAttribute name]

                set hasAttrMessSort [$m hasAttribute messageSort]

                if {$hasAttrMessSort} {
                  if {[$m getAttribute messageSort] eq "reply"} {
                    set messageSort reply
                  } else {
                      if {[$m getAttribute messageSort] eq "createMessage"} {
                        set messageSort createMessage

                      } else {
                        #not expected
                        set messageSort synchCall
                      }
                  }
                } else {
                  # default case 'synchCall' not defined in XMI
                  set messageSort synchCall
                }



								#set callLevel 1

                # call between lifelines:
                if {$sourceName ne $targetName} {
                  set callLevel 0
                # call on the same lifeline:
                } else {
                  # in case that callee (called feature) <old> is equal to caller (calling feature) <new>:
                  # set callLevel 0
                  # and set counter += 1
                  # TODO!!!
                }

                #puts callLevelXX$callLevel
                #puts $accessorTag

                switch $messageSort {
                  reply {
                    # messageText is ordered as follows (according to http://sdedit.sourceforge.net/enter_text/index.html):
                    # <caller>[<s>]:<answer>=<callee>[m].<message>
                    # in our semantic (calling object = source, called feature = target):
                    # <sourceName>[<s>]:<answer>=<targetName>[m].<message>
                    append messageText [lindex $messageSent 0] "\[${callLevel}\]" ":" $mName "=" [lindex $messageSent 1] "." [lindex $messageSent 2]
                    append msText $messageText " \n"
                    set messageSent ""
                    #set tmpCalleeOld ""
                  }
                  createMessage {
                    # check lifelines and insert a '/' before the newly instantiated object
                    # only set "/" before object (i.e. at the beginning of line, after \n)

                    :lifelineInput set [string map "\\n$targetName \\n/$targetName" [:lifelineInput get]]
                    append messageText $sourceName ":" $targetName ".new"
                    append msText $messageText " \n"
                    ## added: (??)
                    set messageSent ""
                  }
                  synchCall {
                    #lappend messageSent $sourceName $targetName $mName
                    # argsText added
                    lappend messageSent $sourceName $targetName "${mName}${argsText} ${accessorTag}"
                    #set tmpCalleeOld ${mName}

                    #puts oldCallee:__$tmpCalleeOld
                    #puts caller:__$callerName
                    #puts newCallee:__$calleeName

                    if {$tmpCalleeOld eq $callerName} {
                      #puts CascadingCalls!!!
                      set callLevel 0
                      #puts callLevel$callLevel
                      incr callLevelCounter
                      #puts callLevelCounter$callLevelCounter
                    } else {
                      set callLevel 1
                    }
                    #puts ----------------

                    set tmpCalleeOld $calleeName
                  }
                  default {
                    #puts "Error: messageSort not identified"
                  }
                }
              }
            }
          }
        }
      }
      return $msText
    }

    # Create file with given name and content
    :protected method createFile {name text} {
      set path $name.sd
      set f $path
      set file [open $f "w"]
      puts $file $text
      close $file
      puts "sd file $path created"
      return $path
    }
  }
}

# works:
#set f exampleModel1.xmi
#set f exampleModel2.xmi
#set f large.xmi

#[::QXP::QsdExporter new] printDiagram $f test123 png
